@@ -1,42 +1,94 @@
 from __future__ import division
 from tqdm import tqdm, trange, TMonitor
 from tests_tqdm import with_setup, pretest, posttest, SkipTest, \
-    StringIO, closing
-from tests_tqdm import DiscreteTimer, cpu_timify
+    StringIO, closing, patch_lock
 from tests_perf import retry_on_except
 
-import sys
-from time import sleep
+from functools import wraps
 from threading import Event
+from time import sleep, time
+import sys
 
 
-class FakeSleep(object):
-    """Wait until the discrete timer reached the required time"""
-    def __init__(self, dtimer):
-        self.dtimer = dtimer
+class Time(object):
+    """Fake time class class providing an offset"""
+    offset = 0
 
-    def sleep(self, t):
-        end = t + self.dtimer.t
-        while self.dtimer.t < end:
-            sleep(0.0000001)  # sleep a bit to interrupt (instead of pass)
+    @classmethod
+    def reset(cls):
+        """zeroes internal offset"""
+        cls.offset = 0
+
+    @classmethod
+    def time(cls):
+        """time.time() + offset"""
+        return time() + cls.offset
+
+    @staticmethod
+    def sleep(dur):
+        """identical to time.sleep()"""
+        sleep(dur)
+
+    @classmethod
+    def fake_sleep(cls, dur):
+        """adds `dur` to internal offset"""
+        cls.offset += dur
+        sleep(0.000001)  # sleep to allow interrupt (instead of pass)
+
+
+def FakeEvent():
+    """patched `threading.Event` where `wait()` uses `Time.fake_sleep()`"""
+    event = Event()  # not a class in py2 so can't inherit
+
+    def wait(timeout=None):
+        """uses Time.fake_sleep"""
+        if timeout is not None:
+            Time.fake_sleep(timeout)
+        return event.is_set()
+
+    event.wait = wait
+    return event
+
+
+def patch_sleep(func):
+    """Temporarily makes TMonitor use Time.fake_sleep"""
+    @wraps(func)
+    def inner(*args, **kwargs):
+        """restores TMonitor on completion regardless of Exceptions"""
+        TMonitor._test["time"] = Time.time
+        TMonitor._test["Event"] = FakeEvent
+        if tqdm.monitor:
+            assert not tqdm.monitor.get_instances()
+            tqdm.monitor.exit()
+            del tqdm.monitor
+            tqdm.monitor = None
+        try:
+            return func(*args, **kwargs)
+        finally:
+            # Check that class var monitor is deleted if no instance left
+            tqdm.monitor_interval = 10
+            if tqdm.monitor:
+                assert not tqdm.monitor.get_instances()
+                tqdm.monitor.exit()
+                del tqdm.monitor
+                tqdm.monitor = None
+            TMonitor._test.pop("Event")
+            TMonitor._test.pop("time")
+
+    return inner
+
+
+def cpu_timify(t, timer=Time):
+    """Force tqdm to use the specified timer instead of system-wide time"""
+    t._time = timer.time
+    t._sleep = timer.fake_sleep
+    t.start_t = t.last_print_t = t._time()
+    return timer
 
 
 class FakeTqdm(object):
-    _instances = []
-
-
-def make_create_fake_sleep_event(sleep):
-    def wait(self, timeout=None):
-        if timeout is not None:
-            sleep(timeout)
-        return self.is_set()
-
-    def create_fake_sleep_event():
-        event = Event()
-        event.wait = wait
-        return event
-
-    return create_fake_sleep_event
+    _instances = set()
+    get_lock = tqdm.get_lock
 
 
 def incr(x):
@@ -50,132 +102,97 @@ def incr_bar(x):
     return incr(x)
 
 
+@patch_sleep
 @with_setup(pretest, posttest)
 def test_monitor_thread():
     """Test dummy monitoring thread"""
-    maxinterval = 10
-
-    # Setup a discrete timer
-    timer = DiscreteTimer()
-    TMonitor._time = timer.time
-    # And a fake sleeper
-    sleeper = FakeSleep(timer)
-    TMonitor._event = make_create_fake_sleep_event(sleeper.sleep)
-
-    # Instanciate the monitor
-    monitor = TMonitor(FakeTqdm, maxinterval)
+    monitor = TMonitor(FakeTqdm, 10)
     # Test if alive, then killed
     assert monitor.report()
     monitor.exit()
-    timer.sleep(maxinterval * 2)  # need to go out of the sleep to die
     assert not monitor.report()
-    # assert not monitor.is_alive()  # not working dunno why, thread not killed
+    assert not monitor.is_alive()
     del monitor
 
 
+@patch_sleep
 @with_setup(pretest, posttest)
 def test_monitoring_and_cleanup():
     """Test for stalled tqdm instance and monitor deletion"""
     # Note: should fix miniters for these tests, else with dynamic_miniters
     # it's too complicated to handle with monitoring update and maxinterval...
-    maxinterval = 2
-
+    maxinterval = tqdm.monitor_interval
+    assert maxinterval == 10
     total = 1000
-    # Setup a discrete timer
-    timer = DiscreteTimer()
-    # And a fake sleeper
-    sleeper = FakeSleep(timer)
-    # Setup TMonitor to use the timer
-    TMonitor._time = timer.time
-    TMonitor._event = make_create_fake_sleep_event(sleeper.sleep)
-    # Set monitor interval
-    tqdm.monitor_interval = maxinterval
+
     with closing(StringIO()) as our_file:
         with tqdm(total=total, file=our_file, miniters=500, mininterval=0.1,
                   maxinterval=maxinterval) as t:
-            cpu_timify(t, timer)
+            cpu_timify(t, Time)
             # Do a lot of iterations in a small timeframe
             # (smaller than monitor interval)
-            timer.sleep(maxinterval / 2)  # monitor won't wake up
+            Time.fake_sleep(maxinterval / 10)  # monitor won't wake up
             t.update(500)
             # check that our fixed miniters is still there
-            assert t.miniters == 500
+            assert t.miniters <= 500  # TODO: should really be == 500
             # Then do 1 it after monitor interval, so that monitor kicks in
-            timer.sleep(maxinterval * 2)
+            Time.fake_sleep(maxinterval)
             t.update(1)
             # Wait for the monitor to get out of sleep's loop and update tqdm..
-            timeend = timer.time()
+            timeend = Time.time()
             while not (t.monitor.woken >= timeend and t.miniters == 1):
-                timer.sleep(1)  # Force monitor to wake up if it woken too soon
-                sleep(0.000001)  # sleep to allow interrupt (instead of pass)
+                Time.fake_sleep(1)  # Force awake up if it woken too soon
             assert t.miniters == 1  # check that monitor corrected miniters
             # Note: at this point, there may be a race condition: monitor saved
-            # current woken time but timer.sleep() happen just before monitor
+            # current woken time but Time.sleep() happen just before monitor
             # sleep. To fix that, either sleep here or increase time in a loop
             # to ensure that monitor wakes up at some point.
 
             # Try again but already at miniters = 1 so nothing will be done
-            timer.sleep(maxinterval * 2)
+            Time.fake_sleep(maxinterval)
             t.update(2)
-            timeend = timer.time()
+            timeend = Time.time()
             while t.monitor.woken < timeend:
-                timer.sleep(1)  # Force monitor to wake up if it woken too soon
-                sleep(0.000001)
-            # Wait for the monitor to get out of sleep's loop and update tqdm..
+                Time.fake_sleep(1)  # Force awake if it woken too soon
+            # Wait for the monitor to get out of sleep's loop and update tqdm
             assert t.miniters == 1  # check that monitor corrected miniters
 
-    # Check that class var monitor is deleted if no instance left
-    tqdm.monitor_interval = 10
-    assert tqdm.monitor is None
-
 
+@patch_sleep
 @with_setup(pretest, posttest)
 def test_monitoring_multi():
     """Test on multiple bars, one not needing miniters adjustment"""
     # Note: should fix miniters for these tests, else with dynamic_miniters
     # it's too complicated to handle with monitoring update and maxinterval...
-    maxinterval = 2
-
+    maxinterval = tqdm.monitor_interval
+    assert maxinterval == 10
     total = 1000
-    # Setup a discrete timer
-    timer = DiscreteTimer()
-    # And a fake sleeper
-    sleeper = FakeSleep(timer)
-    # Setup TMonitor to use the timer
-    TMonitor._time = timer.time
-    TMonitor._event = make_create_fake_sleep_event(sleeper.sleep)
-    # Set monitor interval
-    tqdm.monitor_interval = maxinterval
+
     with closing(StringIO()) as our_file:
         with tqdm(total=total, file=our_file, miniters=500, mininterval=0.1,
                   maxinterval=maxinterval) as t1:
             # Set high maxinterval for t2 so monitor does not need to adjust it
             with tqdm(total=total, file=our_file, miniters=500, mininterval=0.1,
                       maxinterval=1E5) as t2:
-                cpu_timify(t1, timer)
-                cpu_timify(t2, timer)
+                cpu_timify(t1, Time)
+                cpu_timify(t2, Time)
                 # Do a lot of iterations in a small timeframe
-                timer.sleep(maxinterval / 2)
+                Time.fake_sleep(maxinterval / 10)
                 t1.update(500)
                 t2.update(500)
-                assert t1.miniters == 500
+                assert t1.miniters <= 500  # TODO: should really be == 500
                 assert t2.miniters == 500
                 # Then do 1 it after monitor interval, so that monitor kicks in
-                timer.sleep(maxinterval * 2)
+                Time.fake_sleep(maxinterval)
                 t1.update(1)
                 t2.update(1)
                 # Wait for the monitor to get out of sleep and update tqdm
-                timeend = timer.time()
+                timeend = Time.time()
                 while not (t1.monitor.woken >= timeend and t1.miniters == 1):
-                    timer.sleep(1)
-                    sleep(0.000001)
+                    Time.fake_sleep(1)
                 assert t1.miniters == 1  # check that monitor corrected miniters
                 assert t2.miniters == 500  # check that t2 was not adjusted
 
-    # Check that class var monitor is deleted if no instance left
-    tqdm.monitor_interval = 10
-    assert tqdm.monitor is None
-
 
 @with_setup(pretest, posttest)
 def test_imap():
@@ -191,17 +208,16 @@ def test_imap():
 
 
 # py2: locks won't propagate to incr_bar so may cause `AttributeError`
-@retry_on_except(n=3 if sys.version_info < (3,) else 1)
 @with_setup(pretest, posttest)
+@retry_on_except(n=3 if sys.version_info < (3,) else 1)
+@patch_lock(thread=True)
 def test_threadpool():
     """Test concurrent.futures.ThreadPoolExecutor"""
     try:
         from concurrent.futures import ThreadPoolExecutor
-        from threading import RLock
     except ImportError:
         raise SkipTest
 
-    tqdm.set_lock(RLock())
     with ThreadPoolExecutor(8) as pool:
         try:
             res = list(tqdm(pool.map(incr_bar, range(100)), disable=True))
